/* vim: set ai ts=8 sw=4 sts=4 noet: */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "liteamp.h"
#include "playlist.h"

#include "playlist-dialog.h"
#include "location-dialog.h"
#include "track-dialog.h"
#include "prefs-dialog.h"
#include "util.h"

void
on_add_playlist_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    PlaylistDialog *dialog;
    dialog = playlist_dialog_new(get_window(app));
    playlist_dialog_show(dialog, NULL);
}


void
on_remove_playlist_item_activate(GtkMenuItem * menuitem,
				 gpointer user_data)
{
    LaPlaylist *playlist;
    GtkResponseType response;

    playlist = playlist_get_playlist(get_playlist(app));
    if (!playlist) {
	la_message_box(get_window(app), _("Choose a playlist to delete!"));
	return;
    }

    response = la_prompt_boxv(get_window(app),
			      GTK_MESSAGE_QUESTION,
			      GTK_BUTTONS_YES_NO,
			      _("Remove the playlist \"%s\"?"),
			      la_playlist_get_name(playlist));

    if (response = GTK_RESPONSE_YES) {
	sidebar_remove_playlist(get_sidebar(app), playlist);
    }
}


void on_add_file_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    GtkFileSelection *fs;
    GtkResponseType response;
    gchar **selections;
    gchar **filename;
    gchar last_dir[PATH_MAX];

    g_strlcpy(last_dir, prefs.last_dir, sizeof(last_dir));
    if (!la_str_has_suffix_case(prefs.last_dir, G_DIR_SEPARATOR_S))
	g_strlcat(last_dir, "/", sizeof(last_dir));

    fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Add Files")));
    gtk_file_selection_set_select_multiple(fs, TRUE);
    gtk_file_selection_set_filename(fs, last_dir);
    gtk_widget_show_all(GTK_WIDGET(fs));

    response = gtk_dialog_run(GTK_DIALOG(fs));

    if (response == GTK_RESPONSE_OK) {
	selections = gtk_file_selection_get_selections(fs);

	filename = selections;
	while (*filename) {
	    playlist_add_file(get_playlist(app), *filename++, TRUE);
	}

	g_free(prefs.last_dir);
	prefs.last_dir = g_path_get_dirname(*selections);

	g_strfreev(selections);

	playlist_write(get_playlist(app));
    }

    gtk_widget_destroy(GTK_WIDGET(fs));
}


void
on_add_directory_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    GtkFileSelection *fs;
    GtkResponseType response;
    const gchar *filename;
    gchar last_dir[PATH_MAX];

    g_strlcpy(last_dir, prefs.last_dir, sizeof(last_dir));
    if (!la_str_has_suffix_case(prefs.last_dir, G_DIR_SEPARATOR_S))
	g_strlcat(last_dir, "/", sizeof(last_dir));

    fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Add Directories")));
    gtk_file_selection_set_select_multiple(fs, FALSE);
    gtk_file_selection_set_filename(fs, last_dir);
    gtk_widget_show_all(GTK_WIDGET(fs));

    // FIXME: just work-around
    // GTK has no directory selection dialog :(
    gtk_widget_hide(gtk_widget_get_parent(fs->file_list));

    response = gtk_dialog_run(GTK_DIALOG(fs));

    if (response == GTK_RESPONSE_OK) {
	filename = gtk_file_selection_get_filename(fs);
	playlist_add_directory(get_playlist(app), filename, TRUE, TRUE);

	g_free(prefs.last_dir);
	prefs.last_dir = g_path_get_dirname(filename);

	playlist_write(get_playlist(app));
    }

    gtk_widget_destroy(GTK_WIDGET(fs));
}


void
on_add_location_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    LocationDialog *dialog = location_dialog_new(get_window(app));
    location_dialog_show(dialog, "");
}


void on_import_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    GtkFileSelection *fs;
    GtkResponseType response;
    const gchar *filename;
    gchar last_dir[PATH_MAX];

    g_strlcpy(last_dir, prefs.last_dir, sizeof(last_dir));
    if (!la_str_has_suffix_case(prefs.last_dir, G_DIR_SEPARATOR_S))
	g_strlcat(last_dir, "/", sizeof(last_dir));

    fs = GTK_FILE_SELECTION(gtk_file_selection_new
			    (_("Select .pls Files to Import...")));
    gtk_file_selection_set_select_multiple(fs, FALSE);
    gtk_file_selection_set_filename(fs, last_dir);
    gtk_widget_show_all(GTK_WIDGET(fs));

    response = gtk_dialog_run(GTK_DIALOG(fs));

    if (response == GTK_RESPONSE_OK) {
	filename = gtk_file_selection_get_filename(fs);
	playlist_import_pls(get_playlist(app), filename);

	g_free(prefs.last_dir);
	prefs.last_dir = g_path_get_dirname(filename);
    }

    gtk_widget_destroy(GTK_WIDGET(fs));
}


void on_export_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    LaPlaylist *playlist;
    gchar *title;
    GtkFileSelection *fs;
    GtkResponseType response;
    const gchar *filename;

    playlist = playlist_get_playlist(get_playlist(app));
    if (playlist) {
	title =
	    g_strdup_printf(_("Export the playlist \"%s\" as..."),
			    la_playlist_get_name(playlist));
	filename =
	    g_strdup_printf("%s.pls", la_playlist_get_name(playlist));
    } else {
	title = g_strdup(_("Export the library as..."));
	filename = g_strdup("liteamp-library.pls");
    }

    fs = GTK_FILE_SELECTION(gtk_file_selection_new(title));
    gtk_file_selection_set_filename(fs, filename);
    gtk_file_selection_set_select_multiple(fs, FALSE);
    gtk_widget_show_all(GTK_WIDGET(fs));

    response = gtk_dialog_run(GTK_DIALOG(fs));

    if (response == GTK_RESPONSE_OK) {
	filename = gtk_file_selection_get_filename(fs);
	playlist_export_pls(get_playlist(app), filename);

	g_free(prefs.last_dir);
	prefs.last_dir = g_path_get_dirname(filename);
    }

    gtk_widget_destroy(GTK_WIDGET(fs));
    g_free(title);
}


void on_quit_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    gtk_main_quit();
}


void on_cut_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_cut_selection(get_playlist(app));
}


void on_copy_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_copy_selection(get_playlist(app));
}


void on_paste_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_paste_selection(get_playlist(app));
}


void on_clear_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_clear_selection(get_playlist(app));
}


void
on_select_all_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_select_all(get_playlist(app));
}


void
on_select_none_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_select_none(get_playlist(app));
}


void
on_invert_selection_item_activate(GtkMenuItem * menuitem,
				  gpointer user_data)
{
    playlist_invert_selection(get_playlist(app));
}


void
on_properties_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    if (sidebar_has_focus(get_sidebar(app)))
	on_playlist_properties_item_activate(menuitem, user_data);
    else
	on_sidebar_properties_item_activate(menuitem, user_data);
}

void
on_preferences_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    PrefsDialog *dialog;
    dialog = prefs_dialog_new(get_window(app));
    prefs_dialog_show(dialog, &prefs);
}


void
on_configure_effects_item_activate(GtkMenuItem * menuitem,
				   gpointer user_data)
{
    //TODO:extracts equalizer from effect dialog
    effect_show_dialog(app);
}


void on_previous_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_previous(get_playlist(app));
}


void on_start_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    player_start(get_player(app));
}


void on_pause_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    gboolean pause = LA_GET_CHECK_MENU_ITEM(menuitem);
    if (player_is_paused(get_player(app)) == pause)
	return;
    player_pause(get_player(app));
}


void on_stop_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    player_stop(get_player(app));
}


void on_next_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_next(get_playlist(app));
}


void on_back_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    player_back(get_player(app));
}


void on_forward_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    player_forward(get_player(app));
}


void on_shuffle_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.shuffle = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_shuffle(app);
    playlist_shuffle(get_playlist(app));
}


void on_repeat_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.repeat = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_repeat(app);
}


void
on_use_effects_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.use_effects = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_use_effects(app);
}


void on_volume_up_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    mixer_volume_up(get_mixer(app));
    update_ui_volume(app);
}


void
on_volume_down_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    mixer_volume_down(get_mixer(app));
    update_ui_volume(app);
}


void on_mute_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.mute = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_mute(app);
}


void on_compact_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.compact = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_layout(app);
}


void
on_visualizer_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.visualizer = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_layout(app);
}


void on_sidebar_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.sidebar = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_layout(app);
}


void on_playlist_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.playlist = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_layout(app);
}


void on_statusbar_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    prefs.statusbar = LA_GET_CHECK_MENU_ITEM(menuitem);
    update_ui_layout(app);
}


void on_jump_to_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_jump_to(get_playlist(app));
}

void on_locate_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    player_locate(get_player(app));
}

void
on_view_refresh_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    playlist_refresh(get_playlist(app));
}


void on_about_item_activate(GtkMenuItem * menuitem, gpointer user_data)
{
    about_dialog_show();
}


gchar *on_position_hscale_format_value(GtkScale * scale,
				       gdouble value, gpointer user_data)
{
    return la_format_time_str_new((gulong) value);
}


void on_position_hscale_value_changed(GtkRange * range, gpointer user_data)
{
    decoder_seek(gtk_range_get_value(range));
    update_ui_position(app);
}


void on_previous_btn_clicked(GtkButton * button, gpointer user_data)
{
    playlist_previous(get_playlist(app));

}


void on_start_btn_clicked(GtkButton * button, gpointer user_data)
{
    player_start(get_player(app));
}


void on_pause_btn_clicked(GtkButton * button, gpointer user_data)
{
    gboolean pause = LA_GET_TOGGLE_BUTTON(button);
    if (player_is_paused(get_player(app)) == pause)
	return;
    player_pause(get_player(app));
}


void on_stop_btn_clicked(GtkButton * button, gpointer user_data)
{
    player_stop(get_player(app));
}


void on_next_btn_clicked(GtkButton * button, gpointer user_data)
{
    playlist_next(get_playlist(app));
}


void on_shuffle_btn_clicked(GtkButton * button, gpointer user_data)
{
    prefs.shuffle = LA_GET_TOGGLE_BUTTON(button);
    update_ui_shuffle(app);
    playlist_shuffle(get_playlist(app));
}


void on_repeat_btn_clicked(GtkButton * button, gpointer user_data)
{
    prefs.repeat = LA_GET_TOGGLE_BUTTON(button);
    update_ui_repeat(app);
}



void on_mute_btn_clicked(GtkButton * button, gpointer user_data)
{
    prefs.mute = LA_GET_TOGGLE_BUTTON(button);
    update_ui_mute(app);
}


void on_volume_hscale_value_changed(GtkRange * range, gpointer user_data)
{
    mixer_set_volume(get_mixer(app), gtk_range_get_value(range));
    update_ui_volume(app);
}

void
on_sidebar_properties_item_activate(GtkMenuItem * menuitem,
				    gpointer user_data)
{
    LaPlaylist *playlist;
    PlaylistDialog *dialog;
    playlist = sidebar_get_cursor_playlist(get_sidebar(app));
    if (playlist) {
	dialog = playlist_dialog_new(get_window(app));
	playlist_dialog_show(dialog, playlist);
    }
}

void
on_playlist_properties_item_activate(GtkMenuItem * menuitem,
				     gpointer user_data)
{
    LaTrack *track;
    TrackDialog *dialog;
    track = playlist_get_cursor_track(get_playlist(app));
    if (track) {
	dialog = track_dialog_new(get_window(app));
	track_dialog_show(dialog, track);
    }
}

