/* vim: set ai ts=8 sw=4 sts=4 noet: */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "liteamp.h"
#include "track-dialog.h"
#include "util.h"

/* private-----------------------------------------------*/

struct _TrackDialog {
    GObject parent;

    GladeXML *gladexml;
    GtkWidget *dialog;		//GtkDialog
    GtkWidget *title_entry;	//GtkEntry
    GtkWidget *genre_entry;	//GtkEntry
    GtkWidget *artist_entry;	//GtkEntry
    GtkWidget *album_entry;	//GtkEntry
    GtkWidget *year_entry;	//GtkEntry
    GtkWidget *track_entry;	//GtkEntry
    GtkWidget *comment_entry;	//GtkEntry
    GtkWidget *rating_hscale;	//GtkHScale
    GtkWidget *location_value_label;	//GtkLabel
    GtkWidget *quality_value_label;	//GtkLabel
    GtkWidget *time_value_label;	//GtkLabel
    GtkWidget *size_value_label;	//GtkLabel
    GtkWidget *date_value_label;	//GtkLabel

    LaTrack *track;
};

struct _TrackDialogClass {
    GObjectClass parent_class;
};

static void update_data(TrackDialog * self, gboolean save)
{
    if (save) {
	la_track_set_title(self->track,
			   gtk_entry_get_text(GTK_ENTRY
					      (self->title_entry)));
	la_track_set_genre(self->track,
			   gtk_entry_get_text(GTK_ENTRY
					      (self->genre_entry)));
	la_track_set_artist(self->track,
			    gtk_entry_get_text(GTK_ENTRY
					       (self->artist_entry)));
	la_track_set_album(self->track,
			   gtk_entry_get_text(GTK_ENTRY
					      (self->album_entry)));
	la_track_set_year(self->track,
			  gtk_entry_get_text(GTK_ENTRY(self->year_entry)));
	la_track_set_track(self->track,
			   gtk_entry_get_text(GTK_ENTRY
					      (self->track_entry)));
	la_track_set_comment(self->track,
			     gtk_entry_get_text(GTK_ENTRY
						(self->comment_entry)));
	la_track_set_rating(self->track,
			    gtk_range_get_value(GTK_RANGE
						(self->rating_hscale)));
    } else {
	gtk_entry_set_text(GTK_ENTRY(self->title_entry),
			   la_track_get_title(self->track));
	gtk_entry_set_text(GTK_ENTRY(self->genre_entry),
			   la_track_get_genre(self->track));
	gtk_entry_set_text(GTK_ENTRY(self->artist_entry),
			   la_track_get_artist(self->track));
	gtk_entry_set_text(GTK_ENTRY(self->album_entry),
			   la_track_get_album(self->track));
	gtk_entry_set_text(GTK_ENTRY(self->year_entry),
			   la_track_get_year(self->track));
	gtk_entry_set_text(GTK_ENTRY(self->track_entry),
			   la_track_get_track(self->track));
	gtk_entry_set_text(GTK_ENTRY(self->comment_entry),
			   la_track_get_comment(self->track));
	gtk_range_set_value(GTK_RANGE(self->rating_hscale),
			    la_track_get_rating(self->track));
	gtk_label_set_text(GTK_LABEL(self->location_value_label),
			   la_track_get_uri(self->track));
	gtk_label_set_text(GTK_LABEL(self->quality_value_label),
			   la_track_get_quality_str(self->track));
	gtk_label_set_text(GTK_LABEL(self->time_value_label),
			   la_track_get_time_str(self->track));
	gtk_label_set_text(GTK_LABEL(self->size_value_label),
			   la_track_get_size_str(self->track));
	gtk_label_set_text(GTK_LABEL(self->date_value_label),
			   la_track_get_date_str(self->track));
    }
}

static void response_cb(GtkDialog * dialog, gint response,
			TrackDialog * self)
{
    switch (response) {
    case 1:			// up
	return;

    case 2:			// down
	return;

    case GTK_RESPONSE_OK:
	update_data(self, TRUE);
	//@@la_iter_changed(liteamp->library->tracks, self->track);
	break;
    }

    gtk_widget_destroy(GTK_WIDGET(self->dialog));
    return;
}

static gboolean delete_event_cb(GtkWidget * widget, GdkEvent * event,
				TrackDialog * self)
{
    g_object_unref(self);
    return FALSE;
}

/* public-----------------------------------------------*/

TrackDialog *track_dialog_new(GtkWindow * parent)
{
    TrackDialog *self;

    self = g_object_new(TYPE_TRACK_DIALOG, NULL);

    self->gladexml =
	glade_xml_new(GLADE_PATH("/track-dialog.glade"), NULL, NULL);

    self->dialog = glade_xml_get_widget(self->gladexml, "track_dialog");
    self->title_entry =
	glade_xml_get_widget(self->gladexml, "title_entry");
    self->genre_entry =
	glade_xml_get_widget(self->gladexml, "genre_entry");
    self->artist_entry =
	glade_xml_get_widget(self->gladexml, "artist_entry");
    self->album_entry =
	glade_xml_get_widget(self->gladexml, "album_entry");
    self->year_entry = glade_xml_get_widget(self->gladexml, "year_entry");
    self->track_entry =
	glade_xml_get_widget(self->gladexml, "track_entry");
    self->comment_entry =
	glade_xml_get_widget(self->gladexml, "comment_entry");
    self->rating_hscale =
	glade_xml_get_widget(self->gladexml, "rating_hscale");
    self->location_value_label =
	glade_xml_get_widget(self->gladexml, "location_value_label");
    self->quality_value_label =
	glade_xml_get_widget(self->gladexml, "quality_value_label");
    self->time_value_label =
	glade_xml_get_widget(self->gladexml, "time_value_label");
    self->size_value_label =
	glade_xml_get_widget(self->gladexml, "size_value_label");
    self->date_value_label =
	glade_xml_get_widget(self->gladexml, "date_value_label");

    gtk_window_set_transient_for(GTK_WINDOW(self->dialog), parent);
    gtk_dialog_set_default_response(GTK_DIALOG(self->dialog),
				    GTK_RESPONSE_OK);

    g_signal_connect(self->dialog, "response",
		     G_CALLBACK(response_cb), self);
    g_signal_connect(self->dialog, "delete_event",
		     G_CALLBACK(delete_event_cb), self);

    return self;
}

void track_dialog_show(TrackDialog * self, LaTrack * track)
{
    g_return_if_fail(track);

    self->track = track;

    update_data(self, FALSE);

    gtk_widget_show_all(self->dialog);
}

LaTrack *track_dialog_get_track(TrackDialog * self)
{
    return self->track;
}

/* boilerplates -----------------------------------------*/

LA_TYPE_BOILERPLATE(TrackDialog, track_dialog, G_TYPE_OBJECT)

/* track-dialog.c */
